/////////////////////////////////////////////////////////////
// This source code is Copyright (C) David Wilkinson 1996.
// All rights reserved.
//
// For more information,
//
//    Email: davidw@cascade.demon.co.uk
//    URL:   http://www.netlink.co.uk/users/cascade/http/
//
/////////////////////////////////////////////////////////////

// $Id: HTTPConfigurationLog.java,v 1.2 1996/08/04 15:22:33 davidw Exp $
//
// $Log: HTTPConfigurationLog.java,v $
// Revision 1.2  1996/08/04 15:22:33  davidw
// Added copyright
//
// Revision 1.1  1996/06/23 12:31:20  davidw
// Initial revision
//


package uk.co.demon.cascade.http;
import uk.co.demon.cascade.http.*;


import java.util.*;

public class HTTPConfigurationLog implements ConfigurationClass
{
    private static Hashtable objects;

    static
    {
	objects = new Hashtable();
    }

    public boolean configure(Vector tokens) throws InvalidHTTPConfigurationException
    {
	if (tokens.size() != 4)
	{
	    throw (new InvalidHTTPConfigurationException("Invalid Format")); 
	}

	String classname   = (String)tokens.elementAt(1);
	String filename    = (String)tokens.elementAt(2);
	String pattern     = (String)tokens.elementAt(3);

	HTTPSystem.verbose ("Adding log class " + classname + " to " + filename 
			    + " for " + pattern);

        addLog(classname, filename, pattern);
	
	// Don't need to preserve this class
	return false;
    }


    private void addLog(String classname, String filename, String pattern)
    {
	// See if we've already initialised an object for this log

	LogObject o = new LogObject(classname, filename);
	boolean newObject = true;

	LogObject l = null;

	for (Enumeration e = objects.keys(); e.hasMoreElements() ;) 
	{ 
	    l = (LogObject)e.nextElement();
            if (l.equals(o))
	    {
		newObject = false;
		break;
	    }
        }

	if (newObject)
	{
	    // First time for this log type& file

	    o.init();
	    Vector v = new Vector();
	    v.addElement(pattern);
	    objects.put(o, v);
	}
	else
	{
	    Vector v = (Vector) objects.get(l);
	    v.addElement(pattern);
	    objects.put(l, v);
	}
    }

    public static void processLogs(String path, HTTPInformation info)
    {
	// Find all Logger objects that match this path, and process them

	// Really need to abstract this pattern matching out into a
	// separate class at some point

	for (Enumeration en = objects.keys(); en.hasMoreElements() ;) 
	{ 
	    boolean match = false;
	    
	    LogObject obj = (LogObject)en.nextElement();
	    Vector v = (Vector)objects.get(obj);
	    
	    for (Enumeration en_v = v.elements(); en_v.hasMoreElements() ;) 
	    { 
		String pattern = (String)en_v.nextElement();

		int index = pattern.indexOf('*');
		
		if (index == -1)
		{
			// Easy, no wildcards, just check equality

		    if (pattern.equals(path))
		    {
			match = true;
		    }
		}
		else
		{
		    
			// A bit harder, need to process the wildcard
			// Only one wildcard (*) allowed per entry

		    try
		    {
			String a = "";
			String b = "";
	    
			if (index > 0)
			{
			    a = pattern.substring(0, index);
			}
			
			if (index < pattern.length() - 1)
			{
			    b = pattern.substring(index + 1);
			}
			
			if (path.startsWith(a) && path.endsWith(b))
			{
			    match = true;
			}
		    }
		    catch (StringIndexOutOfBoundsException e)
		    {
			match = false;
		    }
		}
		
		if (match)
		{
		    obj.add(info);
		}
	    }
	}
    }
}




