/////////////////////////////////////////////////////////////
// This source code is Copyright (C) David Wilkinson 1996.
// All rights reserved.
//
// For more information,
//
//    Email: davidw@cascade.demon.co.uk
//    URL:   http://www.netlink.co.uk/users/cascade/http/
//
/////////////////////////////////////////////////////////////

// A class used to retrieve an object for the server

// $Id: HTTPFileObject.java,v 1.6 1996/08/04 15:22:33 davidw Exp $
//
// $Log: HTTPFileObject.java,v $
// Revision 1.6  1996/08/04 15:22:33  davidw
// Added copyright
//
// Revision 1.5  1996/06/23 12:41:08  davidw
// Use HTTPInformation instead of HTTPEnvironment
// If no output pipe (eg, for HEAD) then return
//
// Revision 1.4  1996/06/05 18:05:24  davidw
// Handle HEAD requests correctly
//
// Revision 1.3  1996/05/06 15:37:25  davidw
// Made into package uk.co.demon.cascade.http
//
// Revision 1.2  1996/04/28 14:04:42  davidw
// Major rewrite
//
// Revision 1.1  1996/04/28 09:54:13  davidw
// Initial revision
//

package uk.co.demon.cascade.http;
import uk.co.demon.cascade.http.*;



import java.lang.*;
import java.io.*;
import java.util.*;


class HTTPFileObject extends HTTPObject
{
    private String length;
    private String type;

    public HTTPFileObject(String fileName, HTTPInformation i) throws HTTPException
    {
	super(i);

	String method = info.requestMethod;

	try 
	{
	    File file = new File(fileName.toString());
	    
	    if (!file.exists())
	    {
		throw new FileNotFoundException();
	    }

	    length =  new Long(file.length()).toString();
	    type = MimeTypeDictionary.getType(fileName);
	    
	    // Only set up Input stream if method is not HEAD
	    if (!method.equalsIgnoreCase("HEAD"))
	    {
		objectIs = new FileInputStream(fileName.toString());
	    }
	}
	catch (FileNotFoundException e)
	{
	    HTTPException h = new HTTPException("File not found: " + fileName);
	    h.setStatus(HTTPStatus.NOT_FOUND);
	    throw (h);
	}
    }



    public void setupResponse(HTTPResponse response)
    {
	response.addHeader("Content-Length", length);
	response.addHeader("Content-Type", type);
    }


    public void retrieve()
    {
	// Just wait for the pipes to do their stuff
	
	boolean done = false;

	if (pipeOut == null)
	{
	    // There's no pipe to write to, so no more to do
	    done = true;
	}

	while (!done)
	{
	    // Let everyone else have a go while we wait
	    yield();

	    if (pipeOut != null && !pipeOut.stillAlive())
	    {
		done = true;
	    }
       	}

	try
	{
	    if (objectIs != null)
	    {
		objectIs.close();
	    }
	}
	catch (IOException e)
	{
	    HTTPSystem.debug("IOException closing file input stream: " +
			     e.getMessage());
	}
    }
}


