/////////////////////////////////////////////////////////////
// This source code is Copyright (C) David Wilkinson 1996.
// All rights reserved.
//
// For more information,
//
//    Email: davidw@cascade.demon.co.uk
//    URL:   http://www.netlink.co.uk/users/cascade/http/
//
/////////////////////////////////////////////////////////////

// A class used to retrieve an object for the server

// $Id: HTTPObject.java,v 1.5 1996/08/04 15:22:33 davidw Exp $
//
// $Log: HTTPObject.java,v $
// Revision 1.5  1996/08/04 15:22:33  davidw
// Added copyright
//
// Revision 1.4  1996/06/23 12:47:56  davidw
// Pass in HTTPInformation and set number of bytes transferred
// close down streams properly
//
// Revision 1.3  1996/05/06 15:37:25  davidw
// Made into package uk.co.demon.cascade.http
//
// Revision 1.2  1996/04/28 14:05:34  davidw
// Major rewrite
//
// Revision 1.1  1996/04/28 09:54:13  davidw
// Initial revision
//

package uk.co.demon.cascade.http;
import uk.co.demon.cascade.http.*;



import java.lang.*;
import java.io.*;
import java.util.*;


public abstract class HTTPObject extends Thread
{
    // Define the streams it will use to communicate with the outside world
    // These look just like stdin and stout to the object
    protected InputStream stdin;
    protected OutputStream stdout;

    
    // and we must have some streams connected to the object
    protected InputStream objectIs  = null;
    protected OutputStream objectOs = null;

    // Some way to connect all these streams up...
    protected StreamConnector pipeIn  = null;
    protected StreamConnector pipeOut = null;

    // And some useful flags
    protected boolean generatesHeadersFlag = false;
    protected boolean noParseHeadersFlag      = false;

    protected HTTPInformation info;

   
    public HTTPObject(HTTPInformation i)
    {
	stdin = null;
	stdout = null;
	info = i;
    }


    public void setInputStream(InputStream is)
    {
	stdin = is;
    }

    public void setOutputStream(OutputStream os)
    {
	stdout = os;
    }

    public void setHeaderGeneration(boolean b)
    {
	generatesHeadersFlag = b;
    }

    public void setNoParseHeaders(boolean b)
    {
	noParseHeadersFlag = b;
    }

    public boolean generatesHeaders()
    {
	return generatesHeadersFlag;
    }

    public boolean noParseHeaders()
    {
	return noParseHeadersFlag;
    }


    // The main function, to actually run the thread
    public void run()
    {
	if (objectIs != null && stdout != null)
	{
	    pipeOut = new StreamConnector(objectIs, stdout);
	    pipeOut.start();
	}
	
	if (objectOs != null && stdin != null)
	{
	    pipeIn = new StreamConnector(stdin, objectOs);
	    pipeIn.start();
	}
	
	retrieve();
	
	try
	{
	    if (pipeIn != null)
	    {
		pipeIn.kill();
	    }
	    else
	    {
		if (stdin != null)
		{
		    stdin.close();
		}
	    }
		
	}
	catch (IOException e)
	{
	    // Ignore, there's not much we can do
	    HTTPSystem.verbose("IOException Closing Object Streams: " 
			       + e.getMessage());
	}


	try
	{
	    if (pipeOut != null)
	    {
		pipeOut.kill();
		info.bytesTransferred = pipeOut.getCount();
	    }
	    else
	    {
		if (stdout != null)
		{
		    stdout.close();
		}
	    }
	}
	catch (IOException e)
	{
	    // Ignore, there's not much we can do
	    HTTPSystem.verbose("IOException Closing Object Streams: " 
			       + e.getMessage());
	}
    }


    // Abstract method to retrieve the actual object
    public abstract void retrieve();


    public void setupResponse(HTTPResponse response)
    {
    }
}

