/////////////////////////////////////////////////////////////
// This source code is Copyright (C) David Wilkinson 1996.
// All rights reserved.
//
// For more information,
//
//    Email: davidw@cascade.demon.co.uk
//    URL:   http://www.netlink.co.uk/users/cascade/http/
//
/////////////////////////////////////////////////////////////

// A class used to retrieve an object for the server
// just keeps tag of input and output streams really

// $Id: HTTPProcessObject.java,v 1.6 1996/08/04 15:22:33 davidw Exp $
//
// $Log: HTTPProcessObject.java,v $
// Revision 1.6  1996/08/04 15:22:33  davidw
// Added copyright
//
// Revision 1.5  1996/06/23 12:50:20  davidw
// Use HTTPInformation instead of HTTPEnvironment
//
// Revision 1.4  1996/05/06 15:37:25  davidw
// Made into package uk.co.demon.cascade.http
//
// Revision 1.3  1996/05/06 10:42:08  davidw
// Pass environment into exec call
//
// Revision 1.2  1996/04/28 14:02:29  davidw
// Major rewrite.
//
// Revision 1.1  1996/04/28 09:54:13  davidw
// Initial revision
//

package uk.co.demon.cascade.http;
import uk.co.demon.cascade.http.*;



import java.lang.*;
import java.io.*;
import java.util.*;


class HTTPProcessObject extends HTTPObject
{
    private Process proc;

    public HTTPProcessObject(String path, HTTPInformation i) throws HTTPException
    {
	super(i);
	
	try 
	{
	    File file = new File(path.toString());
	    
	    if (!file.exists())
	    {
		throw new FileNotFoundException();
	    }

	    proc = Runtime.getRuntime().exec(path, info.toEnvironment().toArray());
  
	    objectIs = proc.getInputStream();
	    objectOs = proc.getOutputStream();
	}
	catch (FileNotFoundException e)
	{
	    HTTPException h = new HTTPException("File not found: " + path);
	    h.setStatus(HTTPStatus.NOT_FOUND);
	    throw (h);
	}
	catch (IOException e)
	{
	    HTTPException h = new HTTPException("Error during exec: " + path);
	    h.setStatus(HTTPStatus.INTERNAL_ERROR);
	    h.setInfo(e.getMessage());
	    throw (h);
	}
	
	setHeaderGeneration(true);
    }


    public void retrieve()
    {
	try 
	{
	    proc.waitFor();
	}
	catch (InterruptedException e)
	{
	}
    
        // Now just wait for the object to finish & the pipes to do their stuff
	
	boolean done = false;

	while (!done)
	{
	    // Let everyone else have a go while we wait
	    yield();

	    done = true;

	    if (pipeIn != null && pipeIn.stillAlive())
	    {
		done = false;
	    }

	    if (pipeOut != null && pipeOut.stillAlive())
	    {
		done = false;
	    }
	}
    }
}

