/////////////////////////////////////////////////////////////
// This source code is Copyright (C) David Wilkinson 1996.
// All rights reserved.
//
// For more information,
//
//    Email: davidw@cascade.demon.co.uk
//    URL:   http://www.netlink.co.uk/users/cascade/http/
//
/////////////////////////////////////////////////////////////

// $Id: Logger.java,v 1.2 1996/08/04 15:22:33 davidw Exp $
// 
// $Log: Logger.java,v $
// Revision 1.2  1996/08/04 15:22:33  davidw
// Added copyright
//
// Revision 1.1  1996/06/23 12:31:20  davidw
// Initial revision
//

// Base class to maintain logs

package uk.co.demon.cascade.http;
import uk.co.demon.cascade.http.*;

import java.lang.*;
import java.io.*;
import java.util.*;


public class Logger extends Thread
{
    private String filename;
    private RandomAccessFile file;
    private LogQueue queue;

    private static String months[] = 
    {
	"Jan", "Feb", "Mar", "Apr", "May", "Jun", 
	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
    };

    public Logger(String filename)
    {
	super();

	try
	{
	    file = new RandomAccessFile(filename, "rw");
	    file.seek(file.length());   // Position at end of file
	    queue = new LogQueue();
	    setPriority(MIN_PRIORITY); 
	}
	catch (IOException e)
	{
	    HTTPSystem.verbose("Cannot open log file " + filename);
	}
	catch (IllegalArgumentException e)
	{
	    HTTPSystem.verbose("Illegal log thread priority");
	}
    }
    

    public void add(String s)
    {
	if (s != null && file != null)
	{
	    // HTTPSystem.debug(2, "Adding log: " + s);
	    queue.add(s);
	}
    }

    public void run()
    {
	if (file != null)
	{
	    while (true)
	    {
		String s = queue.next();
		
		if (s != null)
		{
		    try
		    {
			file.writeBytes(s + "\n");
		    }
		    catch (IOException e)
		    {
			// Can't do much about this one
			HTTPSystem.verbose("IOException writing to log "
					   + filename + ": " 
					   + e.getMessage());
		    }
		}
		yield();
	    }
	}
    }

    public static String getDateString(Date d)
    {
	// [20/Jun/1996:19:46:19 +0000]
	
	StringBuffer timeStamp = new StringBuffer();

	timeStamp.append("[");
	if (d.getDate() < 10)
	{
	    timeStamp.append(0);
	}
	timeStamp.append(d.getDate());
	timeStamp.append("/");
	timeStamp.append(months[d.getMonth()]);
	timeStamp.append("/");
	int y = 1900 +  d.getYear();
	timeStamp.append(y);
	timeStamp.append(":");
	if (d.getHours() < 10)
	{
	    timeStamp.append(0);
	}
	timeStamp.append(d.getHours());
	timeStamp.append(":");
	if (d.getMinutes() < 10)
	{
	    timeStamp.append(0);
	}
	timeStamp.append(d.getMinutes());
	timeStamp.append(":");
	if (d.getSeconds() < 10)
	{
	    timeStamp.append(0);
	}
	timeStamp.append(d.getSeconds());
	timeStamp.append(" ");

	int tz = d.getTimezoneOffset();

	// XXX This is broken. Set it to zero for now - davidw
	tz = 0;

	String tzsign;
	if (tz < 0)
	{
	    tzsign = "-";
	    tz = -tz;
	}
	else 
	{
	    tzsign = "+";
	}
	int tzhours = tz/60;
	int tzmins = tz - (tzhours * 60);

	
	timeStamp.append(tzsign);
	if (tzhours < 10)
	{
	    timeStamp.append(0);
	}
	timeStamp.append(tzhours);

	if (tzmins < 10)
	{
	    timeStamp.append(0);
	}
	timeStamp.append(tzmins);

	timeStamp.append("]");
	
	return timeStamp.toString();
    }
}

