/////////////////////////////////////////////////////////////
// This source code is Copyright (C) David Wilkinson 1996.
// All rights reserved.
//
// For more information,
//
//    Email: davidw@cascade.demon.co.uk
//    URL:   http://www.netlink.co.uk/users/cascade/http/
//
/////////////////////////////////////////////////////////////


// $Id: PathTranslator.java,v 1.7 1996/08/04 15:22:33 davidw Exp $
//
// $Log: PathTranslator.java,v $
// Revision 1.7  1996/08/04 15:22:33  davidw
// Added copyright
//
// Revision 1.6  1996/06/23 13:19:19  davidw
// Use HTTPInformation instead of HTTPEnvironment
//
// Revision 1.5  1996/06/05 18:10:50  davidw
// Constructor for HTPFileObject now takes in the Environment too
//
// Revision 1.4  1996/06/03 20:19:08  davidw
// HTTPServerExtensionObject constructor now takes environment parameter
//
// Revision 1.3  1996/05/06 15:37:25  davidw
// Made into package uk.co.demon.cascade.http
//
// Revision 1.2  1996/05/06 10:37:19  davidw
// Pass QUERY_STRING to EXEC objects correctly
//
// Revision 1.1  1996/04/28 09:59:30  davidw
// Initial revision
//

package uk.co.demon.cascade.http;
import uk.co.demon.cascade.http.*;


import java.io.*;
import java.util.*;

public class PathTranslator
{
    private HTTPResponse response;
    private HTTPInformation info;
    private String requestedPath;
    private String queryString;
    private String translatedPath;
    private HTTPObject obj;

    private static Vector txObjects;

    static
    {
	txObjects = new Vector();
    }

    public static void addTranslationObject(TranslationObject o)
    {
	txObjects.addElement(o);
    }

    public PathTranslator(HTTPResponse r, HTTPInformation i)
    {
	// Need to know where to set up headers
	response = r;
	
	// And what's happening
	info = i;
    }


    public HTTPObject getHTTPObject(String s) throws HTTPException
    {
	translate(s);
	return obj;
    }

    private void translate(String s) throws HTTPException
    {
	int question = s.indexOf('?'); 
	requestedPath = s;
	queryString = "";

	//HTTPSystem.debug("question: " + question + ", length: " + s.length());
	
	try 
	{
	    if (question >= 0)
	    {

		requestedPath = s.substring(0, question);
		if (question < s.length() - 1)
		{
		    queryString   = s.substring(question + 1);
		}
	    }
	}
	catch (StringIndexOutOfBoundsException e)
	{
	    HTTPSystem.debug("StringIndexOutOfBoundsException: " + e.getMessage());
	    requestedPath = s;
	    queryString = "";
	}
		
	info.scriptName  = requestedPath;
	info.queryString = queryString;
	
	//HTTPSystem.debug("query_string: " + queryString);
	
	int txSize = txObjects.size();
	int txIndex = 0;

	String inTranslation = requestedPath;

	while (txIndex < txSize)
	{
	    TranslationObject o = (TranslationObject) txObjects.elementAt(txIndex);

	    String translated = o.translate(inTranslation);

	    if (translated != null)
	    {
		// We did a translation!

		switch (o.getType())
		{
		  case TranslationObject.MAP:
		    // Mapping, go round again
		    HTTPSystem.verbose ("MAP: " + inTranslation + " -> " + translated);
		    inTranslation = translated;
		    break;

		  case TranslationObject.PASS:
		    // Pass, we've got a file
		    HTTPSystem.verbose ("PASS: " + inTranslation + " -> " + translated);
		    translatedPath = translated;
		    info.translatedPath = translatedPath;
		    obj = new HTTPFileObject(translated, info);
		    obj.setupResponse(response);
		    return;

		  case TranslationObject.EXEC:
		    // Exec
		    HTTPSystem.verbose ("EXEC: " + inTranslation + " -> " + translated);
		    translatedPath = translated;
		    info.translatedPath = translatedPath;

		    // Find the actual executable
		    translatedPath = getExecFile(translatedPath);
		    obj = new HTTPProcessObject(translatedPath, info);
		    obj.setupResponse(response);
		    return;

		  case TranslationObject.CLASSEXEC:
		    // Exec
		    HTTPSystem.verbose ("CLASSEXEC: " + inTranslation + " -> " + translated);
		    translatedPath = translated;
		    info.translatedPath = translatedPath;
		    obj = new HTTPServerExtensionObject(translated, info);
		    obj.setupResponse(response);
		    return;

		  default:
		    HTTPException e = new HTTPException("Invalid Translation Type: " + o.getType());
		    e.setStatus(HTTPStatus.INTERNAL_ERROR);
		    throw(e);
		}
	    }

	    txIndex++;
	}

        // Get the Document Root and map everything onto there for now

	StringBuffer fileName = 
	    new StringBuffer(HTTPConfigurationDocumentRoot.getDocumentRoot());
	    
	fileName.append(inTranslation.replace('/', File.separatorChar));
	translatedPath = fileName.toString();
	HTTPSystem.verbose ("Translated Path = " + translatedPath);
	info.translatedPath = translatedPath;
	obj = new HTTPFileObject(translatedPath, info);
	obj.setupResponse(response);
    }


    private String getExecFile(String s)
    {
	StringTokenizer st = new StringTokenizer(s, File.separator, true);
	StringBuffer filename = new StringBuffer();
	
	while (st.hasMoreTokens())
	{
	    String token = st.nextToken();
	    filename.append(token);
	    HTTPSystem.debug(3, "Building filename: " + filename.toString());
	    File f = new File(filename.toString());
	    if (f.isFile())
	    {
		// Gobble up the rest
		StringBuffer rest = new StringBuffer();

		while (st.hasMoreTokens())
		{
		    rest.append(st.nextToken());
 		}
		
		info.translatedPath = filename.toString();
		
		// Fix up environment variables to remove query string

		try
		{
		    if (rest.length() > 0)
		    {
			info.queryString = rest.toString();

			int index = requestedPath.lastIndexOf(rest.toString());
			if (index > 0)
			{
			    info.scriptName 
				= requestedPath.substring(0, index);
			}
		    }
		}
		catch (StringIndexOutOfBoundsException e)
		{
		    HTTPSystem.debug(3, "StringIndexOutOfBoundsException: " + e.getMessage());
		    return s;
		}

		return filename.toString();
	    }
	}
	return s;
    }
}


