/////////////////////////////////////////////////////////////
// This source code is Copyright (C) David Wilkinson 1996.
// All rights reserved.
//
// For more information,
//
//    Email: davidw@cascade.demon.co.uk
//    URL:   http://www.netlink.co.uk/users/cascade/http/
//
/////////////////////////////////////////////////////////////

// $Id: ServerInfo.java,v 1.2 1996/08/04 15:22:33 davidw Exp $
//
// $Log: ServerInfo.java,v $
// Revision 1.2  1996/08/04 15:22:33  davidw
// Added copyright
//
// Revision 1.1  1996/06/23 12:37:04  davidw
// Initial revision
//

import uk.co.demon.cascade.http.*;

import java.io.*;
import java.net.*;
import java.util.*;

public class ServerInfo implements HTTPServerExtension
{
    InputStream is;
    OutputStream os;
    HTTPEnvironment env;

    public void init(HTTPEnvironment httpEnv, InputStream in, OutputStream out)
    {
	env = httpEnv;
	is = in;
	os = out;
    }

    public void run()
    {
	try
	{
	    generateOutput();
	}
	catch (HTTPException e)
	{
	    // There was an error.
	}
    }
    
    private void generateOutput() throws HTTPException
    {
	HTTPSystem.output(os, "Content-Type: text/html\n\n");
	HTTPSystem.output(os, "<html>\n");
	HTTPSystem.output(os, "<head>\n");
	HTTPSystem.output(os, "<title>Server Info</title>\n");
	HTTPSystem.output(os, "</head>\n");
	HTTPSystem.output(os, "<body>\n");
	HTTPSystem.output(os, "<h1>Server Info</h1>\n");

	printHostname();
	printConnectFrom();
	
	HTTPSystem.output(os, "</body>\n");
	HTTPSystem.output(os, "</html>\n");
    }

    private void printHostname() throws HTTPException
    {
	String hostName;
	Date today = new Date();
	
	try
	{
	    InetAddress localhost = InetAddress.getLocalHost();
	    hostName = localhost.toString();
	}
	catch (UnknownHostException e)
	{
	    hostName = "an unknown host";
	}

	HTTPSystem.output(os, "<p>Server information for " + hostName + 
			  " at " + today.toGMTString() + ".</p>\n");
    }

    private void printConnectFrom() throws HTTPException
    {
	String from;
	
	if (env.get(HTTPEnvironment.remoteHost) != null)
	{
	    from = env.get(HTTPEnvironment.remoteHost);
	}
	else
	{
	    if (env.get(HTTPEnvironment.remoteAddr) != null)
	    {
		from = env.get(HTTPEnvironment.remoteAddr);
	    }
	    else
	    {
		from = "somewhere";
	    }
	}
	HTTPSystem.output(os, "<p>You are connected from " + from + ".</p>\n");
    }
	
}

